use std::{error::Error};
use teloxide::{Bot, types::Message};

#[path="../bot_functions/ban.rs"]
mod ban;
#[path="../bot_functions/mute.rs"]
mod mute;
#[path="../bot_functions/unmute.rs"]
mod unmute;
#[path="../enums/commands.rs"]
mod commands;
#[path= "decorators.rs"]
mod decorators;

pub mod main_handler_endpoints {
    use super::*;

    use super::ban::ban::ban_user;
    use super::mute::mute::mute_user;
    use super::unmute::unmute::unmute;
    use super::commands::enum_commands::Commands;
    use super::decorators::decorators::admin_access;
    use teloxide::{requests::Requester, utils::command::BotCommands};

    pub type CommandHandleResult = Result<(), Box<dyn Error + Send + Sync>>;

    pub async fn facker_endpoint(bot : Bot, msg : Message) -> CommandHandleResult {
        bot.send_message(msg.chat.id, Commands::descriptions().to_string()).await?;
        Ok(())
    }

    pub async fn unmute_endpoint(bot : Bot, msg : Message) -> CommandHandleResult {
        admin_access(bot.clone(), msg.clone(), unmute(
            msg.clone().reply_to_message(),
            &bot,
            msg.chat.id,
        )).await;
        Ok(())
    }

    pub async fn mute_endpoint(
        bot : Bot,
        msg : Message,
        multiplier : i32,
        time : String
    ) -> CommandHandleResult {
        if let 1i32..=10i32 = multiplier{
            admin_access(bot.clone(), msg.clone(), mute_user(
                msg.clone().reply_to_message(),
                multiplier,
                time.to_string(),
                &bot,
                msg.chat.id,
            )).await;
        } else if let 10i32..=i32::MAX = multiplier{
            let except_text = "Вы ввели слишком большой множитель, пожалуйста, \
                    выберите множитель из промежутка [1; 10]";

            bot.send_message(msg.chat.id, except_text)
                .await
                .expect(except_text);
        } else {
            let except_text = "Вы ввели нулевой или отрицательный множитель, пожалуйста, \
                    выберите множитель из промежутка [1; 10]";

            bot.send_message(msg.chat.id, except_text)
                .await
                .expect(except_text);
        }
        Ok(())
    }

    pub async fn tribunal_endpoint(bot : Bot, msg : Message, reason : String) -> CommandHandleResult {
        let reason_result = if reason.is_empty() { None } else { Some(reason) };

        admin_access(bot.clone(), msg.clone(), ban_user(
            msg.reply_to_message(),
            reason_result,
            &bot,
            msg.chat.id,
        )).await;
        Ok(())
    }

}